@@ -26,6 +26,8 @@ void init_t6020_blizzard(void);
 void init_t6020_avalanche(int rev);
 void init_t6021_blizzard(void);
 void init_t6021_avalanche(int rev);
+void init_t6030_sawtooth(void);
+void init_t6030_everest(int rev);
 void init_t6031_sawtooth(void);
 void init_t6031_everest(int rev);
 
@@ -51,7 +53,7 @@ const char *init_cpu(void)
     printf("  CPU part: 0x%x rev: 0x%x\n", part, rev);
 
     if (part >= MIDR_PART_T8015_MONSOON) {
-        /* Enable NEX powergating, the reset cycles might be overriden by chickens */
+        /* Enable NEX powergating, the reset cycles might be overridden by chickens */
         if (!is_ecore()) {
             reg_mask(SYS_IMP_APL_HID13, HID13_RESET_CYCLES_MASK, HID13_RESET_CYCLES(12));
             reg_set(SYS_IMP_APL_HID14, HID14_ENABLE_NEX_POWER_GATING);
@@ -164,6 +166,16 @@ const char *init_cpu(void)
             init_t6021_blizzard();
             break;
 
+        case MIDR_PART_T6030_EVEREST:
+            cpu = "M3 Pro Everest";
+            init_t6030_everest(rev);
+            break;
+
+        case MIDR_PART_T6030_SAWTOOTH:
+            cpu = "M3 Pro Sawtooth";
+            init_t6030_sawtooth();
+            break;
+
         case MIDR_PART_T6031_EVEREST:
             cpu = "M3 Max Everest";
             init_t6031_everest(rev);
