@@ -340,6 +340,8 @@
 #define HID13_POST_ON_CYCLES_MASK   GENMASK(13, 7)
 #define HID13_PRE_CYCLES(x)         ((ULONG(x)) << 14)
 #define HID13_PRE_CYCLES_MASK       GENMASK(17, 14)
+#define HID13_GROUP0_FF0_DELAY(x)   ((ULONG(x)) << 18)
+#define HID13_GROUP0_FF0_DELAY_MASK GENMASK(25, 18)
 #define HID13_GROUP0_FF1_DELAY(x)   ((ULONG(x)) << 26)
 #define HID13_GROUP0_FF1_DELAY_MASK GENMASK(29, 26)
 #define HID13_GROUP0_FF2_DELAY(x)   ((ULONG(x)) << 30)
