@@ -5,6 +5,7 @@
 
 #include "types.h"
 
+/* macOS */
 enum fw_version {
     V_UNKNOWN = 0,
     V12_1,
@@ -48,6 +49,9 @@ extern const struct fw_version_info fw_versions[NUM_FW_VERSIONS];
 
 int firmware_init(void);
 int firmware_set_fdt(void *fdt, int node, const char *prop, const struct fw_version_info *ver);
+bool firmware_iboot_in_range(u32 min[IBOOT_VER_COMP], u32 max[IBOOT_VER_COMP],
+                             u32 this[IBOOT_VER_COMP]);
 bool firmware_sfw_in_range(enum fw_version lower_bound, enum fw_version upper_bound);
+void firmware_parse_version(const char *s, u32 *out);
 
 #endif
