@@ -4,6 +4,7 @@
 #include "kboot_atc.h"
 #include "adt.h"
 #include "devicetree.h"
+#include "malloc.h"
 #include "pmgr.h"
 #include "utils.h"
 
@@ -405,30 +406,29 @@ static void dt_copy_atc_tunables(void *dt, const char *adt_path, const char *dt_
 
     /*
      * For backwards compatibility with downstream drivers copy apple,tunable-common-b to
-     * apple,tunable-common and apple,tunable-common-a to apple,tunable-fuses.
+     * apple,tunable-common.
      * Don't remove this before 2027-01-01.
      */
     int prop_len;
-    const void *tunable_common_a = fdt_getprop(dt, fdt_node, "apple,tunable-common-b", &prop_len);
-    if (!tunable_common_a) {
+    const void *tunable_common_b_fdt =
+        fdt_getprop(dt, fdt_node, "apple,tunable-common-b", &prop_len);
+    if (!tunable_common_b_fdt) {
         printf("kboot: Unable to find apple,tunable-common-b for %s\n", adt_path);
         goto cleanup;
     }
-    ret = fdt_setprop(dt, fdt_node, "apple,tunable-common", tunable_common_a, prop_len);
-    if (ret) {
+
+    void *tunable_common_b = malloc(prop_len);
+    if (!tunable_common_b) {
         printf("kboot: Unable to copy apple,tunable-common-b to apple,tunable-common for %s\n",
                adt_path);
         goto cleanup;
     }
+    memcpy(tunable_common_b, tunable_common_b_fdt, prop_len);
 
-    const void *tunable_common_b = fdt_getprop(dt, fdt_node, "apple,tunable-common-b", &prop_len);
-    if (!tunable_common_b) {
-        printf("kboot: Unable to find apple,tunable-common-b for %s\n", adt_path);
-        goto cleanup;
-    }
-    ret = fdt_setprop(dt, fdt_node, "apple,tunable-fuses", tunable_common_b, prop_len);
+    ret = fdt_setprop(dt, fdt_node, "apple,tunable-common", tunable_common_b, prop_len);
+    free(tunable_common_b);
     if (ret) {
-        printf("kboot: Unable to copy apple,tunable-common-a to apple,tunable-fuses for %s\n",
+        printf("kboot: Unable to copy apple,tunable-common-b to apple,tunable-common for %s\n",
                adt_path);
         goto cleanup;
     }
@@ -445,7 +445,6 @@ cleanup:
         fdt_delprop(dt, fdt_node, atc_tunables[i].fdt_name);
     fdt_delprop(dt, fdt_node, "apple,tunable-common-a");
     fdt_delprop(dt, fdt_node, "apple,tunable-common");
-    fdt_delprop(dt, fdt_node, "apple,tunable-fuses");
 
     printf("FDT: Unable to setup ATC tunables for %s - USB3/Thunderbolt will not work\n", adt_path);
 }
