@@ -53,7 +53,7 @@ typedef s64 ssize_t;
 #define MASK(x)                (BIT(x) - 1)
 #define GENMASK(msb, lsb)      ((BIT((msb + 1) - (lsb)) - 1) << (lsb))
 #define _FIELD_LSB(field)      ((field) & ~(field - 1))
-#define FIELD_PREP(field, val) ((val) * (_FIELD_LSB(field)))
+#define FIELD_PREP(field, val) (((val) * (_FIELD_LSB(field))) & (field))
 #define FIELD_GET(field, val)  (((val) & (field)) / _FIELD_LSB(field))
 
 #ifdef __ASSEMBLER__
